require 'bazaar_voice/version'
require 'rest-client'

module BazaarVoice
  class Api
    class << self
      attr_writer :configuration

      def configuration
        @configuration ||= Configuration.new
      end

      def reset
        @configuration = Configuration.new
      end

      def configure
        yield(configuration)
      end

      # Some metaprogramming to define many similar methods.
      %w(reviews categories stories questions comments answers authors products statistics).each do |resource|
        define_method("get_#{resource}") do |argument = nil|
          get(resource, argument)
        end
      end

      def was_helpful(boolean, content_id)
        post(
          'submitfeedback',
          'ContentType' => 'review',
          'FeedbackType' => 'helpfulness',
          'Vote' => (boolean ? 'Positive' : 'Negative'),
          'ContentId' => content_id
        )
      end

      private

      def rest_client
        @rest_client ||= rest_client_with_logger
      end

      def rest_client_with_logger
        RestClient.log = configuration.logger
        RestClient
      end

      def get(resource, params = nil)
        JSON.parse rest_client.get(uri(resource, params))
      end

      def post(resource, params = nil)
        JSON.parse rest_client.post(uri(resource, params), nil)
      end

      def uri(resource, params = nil)
        "#{configuration.protocol}://#{configuration.host}/data/#{resource}.#{configuration.format}#{query_string(params)}"
      end

      def query_string(params = nil)
        all_params = { passkey: configuration.passkey, apiversion: configuration.apiversion }.merge(params || {})
        '?' + all_params.map { |k, v| "#{k}=#{v}" }.join('&')
      end
    end
  end

  class Configuration
    attr_accessor :passkey, :apiversion, :host, :resource, :protocol, :format, :logger
    def initialize
      @apiversion = '5.4'
      @host = 'api.bazaarvoice.com'
      @protocol = 'https'
      @format = 'json'
      @logger = $stdout
    end
  end

  class Error < StandardError
  end
end
